@@ -26,10 +26,10 @@ from libtiff.tif_lzw import encode as c_encode, decode as c_decode
 def test_encode():
     for arr in [
             numpy.array([7, 7, 7, 8, 8, 7, 7, 6, 6], numpy.uint8),
-            numpy.array(list(range(400000)), numpy.uint8),
+            numpy.array(list(range(400000))).astype(numpy.uint8),
             numpy.array([1, 3, 7, 15, 31, 63], numpy.uint8)]:
 
         rarr = c_encode(arr)
         arr2 = c_decode(rarr, arr.nbytes)
-        assert arr2.nbytes == arr.nbytes and (arr2 == arr).all(),\
+        assert arr2.nbytes == arr.nbytes and (arr2 == arr).all(), \
             repr((arr2, arr))
