@@ -1,12 +1,9 @@
-TIFF_VERSION = 42
-TIFF_BIGTIFF_VERSION = 43
+TIFF_VERSION_CLASSIC = 42
+TIFF_VERSION_BIG = 43
 TIFF_BIGENDIAN = 19789
 TIFF_LITTLEENDIAN = 18761
 MDI_LITTLEENDIAN = 20549
 MDI_BIGENDIAN = 17744
-TIFF_MAGIC_SIZE = 2
-TIFF_VERSION_SIZE = 2
-TIFF_DIROFFSET_SIZE = 4
 TIFFTAG_SUBFILETYPE = 254
 FILETYPE_REDUCEDIMAGE = 1
 FILETYPE_PAGE = 2
@@ -28,6 +25,8 @@ COMPRESSION_CCITT_T6 = 4
 COMPRESSION_LZW = 5
 COMPRESSION_OJPEG = 6
 COMPRESSION_JPEG = 7
+COMPRESSION_T85 = 9
+COMPRESSION_T43 = 10
 COMPRESSION_NEXT = 32766
 COMPRESSION_CCITTRLEW = 32771
 COMPRESSION_PACKBITS = 32773
@@ -45,6 +44,11 @@ COMPRESSION_JBIG = 34661
 COMPRESSION_SGILOG = 34676
 COMPRESSION_SGILOG24 = 34677
 COMPRESSION_JP2000 = 34712
+COMPRESSION_LERC = 34887
+COMPRESSION_LZMA = 34925
+COMPRESSION_ZSTD = 50000
+COMPRESSION_WEBP = 50001
+COMPRESSION_JXL = 50002
 TIFFTAG_PHOTOMETRIC = 262
 PHOTOMETRIC_MINISWHITE = 0
 PHOTOMETRIC_MINISBLACK = 1
@@ -56,6 +60,7 @@ PHOTOMETRIC_YCBCR = 6
 PHOTOMETRIC_CIELAB = 8
 PHOTOMETRIC_ICCLAB = 9
 PHOTOMETRIC_ITULAB = 10
+PHOTOMETRIC_CFA = 32803
 PHOTOMETRIC_LOGL = 32844
 PHOTOMETRIC_LOGLUV = 32845
 TIFFTAG_THRESHHOLDING = 263
@@ -172,6 +177,29 @@ TIFFTAG_YCLIPPATHUNITS = 345
 TIFFTAG_INDEXED = 346
 TIFFTAG_JPEGTABLES = 347
 TIFFTAG_OPIPROXY = 351
+TIFFTAG_GLOBALPARAMETERSIFD = 400
+TIFFTAG_PROFILETYPE = 401
+PROFILETYPE_UNSPECIFIED = 0
+PROFILETYPE_G3_FAX = 1
+TIFFTAG_FAXPROFILE = 402
+FAXPROFILE_S = 1
+FAXPROFILE_F = 2
+FAXPROFILE_J = 3
+FAXPROFILE_C = 4
+FAXPROFILE_L = 5
+FAXPROFILE_M = 6
+TIFFTAG_CODINGMETHODS = 403
+CODINGMETHODS_T4_1D = 2
+CODINGMETHODS_T4_2D = 4
+CODINGMETHODS_T6 = 8
+CODINGMETHODS_T85 = 16
+CODINGMETHODS_T42 = 32
+CODINGMETHODS_T43 = 64
+TIFFTAG_VERSIONYEAR = 404
+TIFFTAG_MODENUMBER = 405
+TIFFTAG_DECODE = 433
+TIFFTAG_IMAGEBASECOLOR = 434
+TIFFTAG_T82OPTIONS = 435
 TIFFTAG_JPEGPROC = 512
 JPEGPROC_BASELINE = 1
 JPEGPROC_LOSSLESS = 14
@@ -189,8 +217,10 @@ TIFFTAG_YCBCRPOSITIONING = 531
 YCBCRPOSITION_CENTERED = 1
 YCBCRPOSITION_COSITED = 2
 TIFFTAG_REFERENCEBLACKWHITE = 532
+TIFFTAG_STRIPROWCOUNTS = 559
 TIFFTAG_XMLPACKET = 700
 TIFFTAG_OPIIMAGEID = 32781
+TIFFTAG_TIFFANNOTATIONDATA = 32932
 TIFFTAG_REFPTS = 32953
 TIFFTAG_REGIONTACKPOINT = 32954
 TIFFTAG_REGIONWARPCORNERS = 32955
@@ -207,8 +237,23 @@ TIFFTAG_PIXAR_FOVCOT = 33304
 TIFFTAG_PIXAR_MATRIX_WORLDTOSCREEN = 33305
 TIFFTAG_PIXAR_MATRIX_WORLDTOCAMERA = 33306
 TIFFTAG_WRITERSERIALNUMBER = 33405
+TIFFTAG_CFAREPEATPATTERNDIM = 33421
+TIFFTAG_CFAPATTERN = 33422
+TIFFTAG_BATTERYLEVEL = 33423
 TIFFTAG_COPYRIGHT = 33432
+TIFFTAG_MD_FILETAG = 33445
+TIFFTAG_MD_SCALEPIXEL = 33446
+TIFFTAG_MD_COLORTABLE = 33447
+TIFFTAG_MD_LABNAME = 33448
+TIFFTAG_MD_SAMPLEINFO = 33449
+TIFFTAG_MD_PREPDATE = 33450
+TIFFTAG_MD_PREPTIME = 33451
+TIFFTAG_MD_FILEUNITS = 33452
 TIFFTAG_RICHTIFFIPTC = 33723
+TIFFTAG_INGR_PACKET_DATA_TAG = 33918
+TIFFTAG_INGR_FLAG_REGISTERS = 33919
+TIFFTAG_IRASB_TRANSORMATION_MATRIX = 33920
+TIFFTAG_MODELTIEPOINTTAG = 33922
 TIFFTAG_IT8SITE = 34016
 TIFFTAG_IT8COLORSEQUENCE = 34017
 TIFFTAG_IT8HEADER = 34018
@@ -227,9 +272,11 @@ TIFFTAG_IT8HCUSAGE = 34030
 TIFFTAG_IT8TRAPINDICATOR = 34031
 TIFFTAG_IT8CMYKEQUIVALENT = 34032
 TIFFTAG_FRAMECOUNT = 34232
+TIFFTAG_MODELTRANSFORMATIONTAG = 34264
 TIFFTAG_PHOTOSHOP = 34377
 TIFFTAG_EXIFIFD = 34665
 TIFFTAG_ICCPROFILE = 34675
+TIFFTAG_IMAGELAYER = 34732
 TIFFTAG_JBIGOPTIONS = 34750
 TIFFTAG_GPSIFD = 34853
 TIFFTAG_FAXRECVPARAMS = 34908
@@ -238,7 +285,15 @@ TIFFTAG_FAXRECVTIME = 34910
 TIFFTAG_FAXDCS = 34911
 TIFFTAG_STONITS = 37439
 TIFFTAG_FEDEX_EDR = 34929
+TIFFTAG_IMAGESOURCEDATA = 37724
 TIFFTAG_INTEROPERABILITYIFD = 40965
+TIFFTAG_GDAL_METADATA = 42112
+TIFFTAG_GDAL_NODATA = 42113
+TIFFTAG_OCE_SCANJOB_DESCRIPTION = 50215
+TIFFTAG_OCE_APPLICATION_SELECTOR = 50216
+TIFFTAG_OCE_IDENTIFICATION_NUMBER = 50217
+TIFFTAG_OCE_IMAGELOGIC_CHARACTERISTICS = 50218
+TIFFTAG_LERC_PARAMETERS = 50674
 TIFFTAG_DNGVERSION = 50706
 TIFFTAG_DNGBACKWARDVERSION = 50707
 TIFFTAG_UNIQUECAMERAMODEL = 50708
@@ -287,6 +342,109 @@ TIFFTAG_ASSHOTICCPROFILE = 50831
 TIFFTAG_ASSHOTPREPROFILEMATRIX = 50832
 TIFFTAG_CURRENTICCPROFILE = 50833
 TIFFTAG_CURRENTPREPROFILEMATRIX = 50834
+TIFFTAG_COLORIMETRICREFERENCE = 50879
+TIFFTAG_CAMERACALIBRATIONSIGNATURE = 50931
+TIFFTAG_PROFILECALIBRATIONSIGNATURE = 50932
+TIFFTAG_ASSHOTPROFILENAME = 50934
+TIFFTAG_NOISEREDUCTIONAPPLIED = 50935
+TIFFTAG_PROFILENAME = 50936
+TIFFTAG_PROFILEHUESATMAPDIMS = 50937
+TIFFTAG_PROFILEHUESATMAPDATA1 = 50938
+TIFFTAG_PROFILEHUESATMAPDATA2 = 50939
+TIFFTAG_PROFILETONECURVE = 50940
+TIFFTAG_PROFILEEMBEDPOLICY = 50941
+TIFFTAG_PROFILECOPYRIGHT = 50942
+TIFFTAG_FORWARDMATRIX1 = 50964
+TIFFTAG_FORWARDMATRIX2 = 50965
+TIFFTAG_PREVIEWAPPLICATIONNAME = 50966
+TIFFTAG_PREVIEWAPPLICATIONVERSION = 50967
+TIFFTAG_PREVIEWSETTINGSNAME = 50968
+TIFFTAG_PREVIEWSETTINGSDIGEST = 50969
+TIFFTAG_PREVIEWCOLORSPACE = 50970
+TIFFTAG_PREVIEWDATETIME = 50971
+TIFFTAG_RAWIMAGEDIGEST = 50972
+TIFFTAG_ORIGINALRAWFILEDIGEST = 50973
+TIFFTAG_SUBTILEBLOCKSIZE = 50974
+TIFFTAG_ROWINTERLEAVEFACTOR = 50975
+TIFFTAG_PROFILELOOKTABLEDIMS = 50981
+TIFFTAG_PROFILELOOKTABLEDATA = 50982
+TIFFTAG_OPCODELIST1 = 51008
+TIFFTAG_OPCODELIST2 = 51009
+TIFFTAG_OPCODELIST3 = 51022
+TIFFTAG_NOISEPROFILE = 51041
+TIFFTAG_DEFAULTUSERCROP = 51125
+TIFFTAG_DEFAULTBLACKRENDER = 51110
+TIFFTAG_BASELINEEXPOSUREOFFSET = 51109
+TIFFTAG_PROFILELOOKTABLEENCODING = 51108
+TIFFTAG_PROFILEHUESATMAPENCODING = 51107
+TIFFTAG_ORIGINALDEFAULTFINALSIZE = 51089
+TIFFTAG_ORIGINALBESTQUALITYFINALSIZE = 51090
+TIFFTAG_ORIGINALDEFAULTCROPSIZE = 51091
+TIFFTAG_NEWRAWIMAGEDIGEST = 51111
+TIFFTAG_RAWTOPREVIEWGAIN = 51112
+TIFFTAG_DEPTHFORMAT = 51177
+TIFFTAG_DEPTHNEAR = 51178
+TIFFTAG_DEPTHFAR = 51179
+TIFFTAG_DEPTHUNITS = 51180
+TIFFTAG_DEPTHMEASURETYPE = 51181
+TIFFTAG_ENHANCEPARAMS = 51182
+TIFFTAG_PROFILEGAINTABLEMAP = 52525
+TIFFTAG_SEMANTICNAME = 52526
+TIFFTAG_SEMANTICINSTANCEID = 52528
+TIFFTAG_MASKSUBAREA = 52536
+TIFFTAG_RGBTABLES = 52543
+TIFFTAG_CALIBRATIONILLUMINANT3 = 52529
+TIFFTAG_COLORMATRIX3 = 52531
+TIFFTAG_CAMERACALIBRATION3 = 52530
+TIFFTAG_REDUCTIONMATRIX3 = 52538
+TIFFTAG_PROFILEHUESATMAPDATA3 = 52537
+TIFFTAG_FORWARDMATRIX3 = 52532
+TIFFTAG_ILLUMINANTDATA1 = 52533
+TIFFTAG_ILLUMINANTDATA2 = 52534
+TIFFTAG_ILLUMINANTDATA3 = 53535
+TIFFTAG_EP_CFAREPEATPATTERNDIM = 33421
+TIFFTAG_EP_CFAPATTERN = 33422
+TIFFTAG_EP_BATTERYLEVEL = 33423
+TIFFTAG_EP_INTERLACE = 34857
+TIFFTAG_EP_IPTC_NAA = 33723
+TIFFTAG_EP_TIMEZONEOFFSET = 34858
+TIFFTAG_EP_SELFTIMERMODE = 34859
+TIFFTAG_EP_FLASHENERGY = 37387
+TIFFTAG_EP_SPATIALFREQUENCYRESPONSE = 37388
+TIFFTAG_EP_NOISE = 37389
+TIFFTAG_EP_FOCALPLANEXRESOLUTION = 37390
+TIFFTAG_EP_FOCALPLANEYRESOLUTION = 37391
+TIFFTAG_EP_FOCALPLANERESOLUTIONUNIT = 37392
+TIFFTAG_EP_IMAGENUMBER = 37393
+TIFFTAG_EP_SECURITYCLASSIFICATION = 37394
+TIFFTAG_EP_IMAGEHISTORY = 37395
+TIFFTAG_EP_EXPOSUREINDEX = 37397
+TIFFTAG_EP_STANDARDID = 37398
+TIFFTAG_EP_SENSINGMETHOD = 37399
+TIFFTAG_EP_EXPOSURETIME = 33434
+TIFFTAG_EP_FNUMBER = 33437
+TIFFTAG_EP_EXPOSUREPROGRAM = 34850
+TIFFTAG_EP_SPECTRALSENSITIVITY = 34852
+TIFFTAG_EP_ISOSPEEDRATINGS = 34855
+TIFFTAG_EP_OECF = 34856
+TIFFTAG_EP_DATETIMEORIGINAL = 36867
+TIFFTAG_EP_COMPRESSEDBITSPERPIXEL = 37122
+TIFFTAG_EP_SHUTTERSPEEDVALUE = 37377
+TIFFTAG_EP_APERTUREVALUE = 37378
+TIFFTAG_EP_BRIGHTNESSVALUE = 37379
+TIFFTAG_EP_EXPOSUREBIASVALUE = 37380
+TIFFTAG_EP_MAXAPERTUREVALUE = 37381
+TIFFTAG_EP_SUBJECTDISTANCE = 37382
+TIFFTAG_EP_METERINGMODE = 37383
+TIFFTAG_EP_LIGHTSOURCE = 37384
+TIFFTAG_EP_FLASH = 37385
+TIFFTAG_EP_FOCALLENGTH = 37386
+TIFFTAG_EP_SUBJECTLOCATION = 37396
+TIFFTAG_RPCCOEFFICIENT = 50844
+TIFFTAG_ALIAS_LAYER_METADATA = 50784
+TIFFTAG_TIFF_RSID = 50908
+TIFFTAG_GEO_METADATA = 50909
+TIFFTAG_EXTRACAMERAPROFILES = 50933
 TIFFTAG_DCSHUESHIFTVALUES = 65535
 TIFFTAG_FAXMODE = 65536
 FAXMODE_CLASSIC = 0
@@ -337,11 +495,30 @@ SGILOGDATAFMT_8BIT = 3
 TIFFTAG_SGILOGENCODE = 65561
 SGILOGENCODE_NODITHER = 0
 SGILOGENCODE_RANDITHER = 1
+TIFFTAG_LZMAPRESET = 65562
+TIFFTAG_PERSAMPLE = 65563
+PERSAMPLE_MERGED = 0
+PERSAMPLE_MULTI = 1
+TIFFTAG_ZSTD_LEVEL = 65564
+TIFFTAG_LERC_VERSION = 65565
+LERC_VERSION_2_4 = 4
+TIFFTAG_LERC_ADD_COMPRESSION = 65566
+LERC_ADD_COMPRESSION_NONE = 0
+LERC_ADD_COMPRESSION_DEFLATE = 1
+LERC_ADD_COMPRESSION_ZSTD = 2
+TIFFTAG_LERC_MAXZERROR = 65567
+TIFFTAG_WEBP_LEVEL = 65568
+TIFFTAG_WEBP_LOSSLESS = 65569
+TIFFTAG_WEBP_LOSSLESS_EXACT = 65571
+TIFFTAG_DEFLATE_SUBCODEC = 65570
+DEFLATE_SUBCODEC_ZLIB = 0
+DEFLATE_SUBCODEC_LIBDEFLATE = 1
 EXIFTAG_EXPOSURETIME = 33434
 EXIFTAG_FNUMBER = 33437
 EXIFTAG_EXPOSUREPROGRAM = 34850
 EXIFTAG_SPECTRALSENSITIVITY = 34852
 EXIFTAG_ISOSPEEDRATINGS = 34855
+EXIFTAG_PHOTOGRAPHICSENSITIVITY = 34855
 EXIFTAG_OECF = 34856
 EXIFTAG_EXIFVERSION = 36864
 EXIFTAG_DATETIMEORIGINAL = 36867
@@ -392,6 +569,61 @@ EXIFTAG_SATURATION = 41993
 EXIFTAG_SHARPNESS = 41994
 EXIFTAG_DEVICESETTINGDESCRIPTION = 41995
 EXIFTAG_SUBJECTDISTANCERANGE = 41996
-EXIFTAG_GAINCONTROL = 41991
-EXIFTAG_GAINCONTROL = 41991
 EXIFTAG_IMAGEUNIQUEID = 42016
+EXIFTAG_SENSITIVITYTYPE = 34864
+EXIFTAG_STANDARDOUTPUTSENSITIVITY = 34865
+EXIFTAG_RECOMMENDEDEXPOSUREINDEX = 34866
+EXIFTAG_ISOSPEED = 34867
+EXIFTAG_ISOSPEEDLATITUDEYYY = 34868
+EXIFTAG_ISOSPEEDLATITUDEZZZ = 34869
+EXIFTAG_OFFSETTIME = 36880
+EXIFTAG_OFFSETTIMEORIGINAL = 36881
+EXIFTAG_OFFSETTIMEDIGITIZED = 36882
+EXIFTAG_TEMPERATURE = 37888
+EXIFTAG_HUMIDITY = 37889
+EXIFTAG_PRESSURE = 37890
+EXIFTAG_WATERDEPTH = 37891
+EXIFTAG_ACCELERATION = 37892
+EXIFTAG_CAMERAELEVATIONANGLE = 37893
+EXIFTAG_CAMERAOWNERNAME = 42032
+EXIFTAG_BODYSERIALNUMBER = 42033
+EXIFTAG_LENSSPECIFICATION = 42034
+EXIFTAG_LENSMAKE = 42035
+EXIFTAG_LENSMODEL = 42036
+EXIFTAG_LENSSERIALNUMBER = 42037
+EXIFTAG_GAMMA = 42240
+EXIFTAG_COMPOSITEIMAGE = 42080
+EXIFTAG_SOURCEIMAGENUMBEROFCOMPOSITEIMAGE = 42081
+EXIFTAG_SOURCEEXPOSURETIMESOFCOMPOSITEIMAGE = 42082
+GPSTAG_VERSIONID = 0
+GPSTAG_LATITUDEREF = 1
+GPSTAG_LATITUDE = 2
+GPSTAG_LONGITUDEREF = 3
+GPSTAG_LONGITUDE = 4
+GPSTAG_ALTITUDEREF = 5
+GPSTAG_ALTITUDE = 6
+GPSTAG_TIMESTAMP = 7
+GPSTAG_SATELLITES = 8
+GPSTAG_STATUS = 9
+GPSTAG_MEASUREMODE = 10
+GPSTAG_DOP = 11
+GPSTAG_SPEEDREF = 12
+GPSTAG_SPEED = 13
+GPSTAG_TRACKREF = 14
+GPSTAG_TRACK = 15
+GPSTAG_IMGDIRECTIONREF = 16
+GPSTAG_IMGDIRECTION = 17
+GPSTAG_MAPDATUM = 18
+GPSTAG_DESTLATITUDEREF = 19
+GPSTAG_DESTLATITUDE = 20
+GPSTAG_DESTLONGITUDEREF = 21
+GPSTAG_DESTLONGITUDE = 22
+GPSTAG_DESTBEARINGREF = 23
+GPSTAG_DESTBEARING = 24
+GPSTAG_DESTDISTANCEREF = 25
+GPSTAG_DESTDISTANCE = 26
+GPSTAG_PROCESSINGMETHOD = 27
+GPSTAG_AREAINFORMATION = 28
+GPSTAG_DATESTAMP = 29
+GPSTAG_DIFFERENTIAL = 30
+GPSTAG_GPSHPOSITIONINGERROR = 31
